using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace VaultAgent
{
    internal class ServerSocket
    {
        public delegate void CallBackDataRecieved(NetworkStream ns, string text);
        public CallBackDataRecieved OnDataRecieved;
        public CallBackDataRecieved OnError;
        public CallBackDataRecieved OnLog;

        #region Private Members
        private TcpListener _listener = null;
        private int _port;
        private bool _listening = false;
        private object _syncRoot = new object();
        #endregion

        #region CTORs
        public bool Listening
        {
            get { return _listening; }
        }
        #endregion

        #region Public Methods
        public void Listen(int iPort)
        {
            _port = iPort;

            try
            {
                lock (_syncRoot)
                {
                    if (OnLog != null)
                        OnLog(null, "TCP listening starting on port " + _port.ToString() + ".");

                    IPAddress ipAddress = IPAddress.Any;
                    _listener = new TcpListener(ipAddress, _port);

                    // fire up the server
                    _listener.Start();
                    // set listening bit
                    _listening = true;
                }

                // Enter the listening loop.
                do
                {
                    // Wait for connection
                    TcpClient newClient = _listener.AcceptTcpClient();
                    if (OnLog != null)
                        OnLog(null, "Connected to new client");

                    // queue a request to take care of the client
                    ThreadPool.QueueUserWorkItem(new WaitCallback(ProcessClient), newClient);
                } while (_listening);
            }
            catch (SocketException se)
            {
                if (_listening)
                {
                    if (OnError != null)
                        OnError(null, "SocketException: " + se.ToString());
                }
            }
            finally
            {
                // shut it down
                StopListening();
            }
        }

        public void StopListening()
        {
            if (_listening)
            {
                lock (_syncRoot)
                {
                    // set listening bit
                    _listening = false;
                    // shut it down
                    _listener.Stop();
                }
                if (OnLog != null)
                    OnLog(null, "Listening stopped");
            }
        }
        #endregion

        #region Private Methods
        private void ProcessClient(object client)
        {
            TcpClient newClient = (TcpClient)client;
            try
            {
                //System.Windows.Forms.MessageBox.Show(newClient.ReceiveBufferSize.ToString()
                //    + " avail " + newClient.Available.ToString());

                // Buffer for reading data

                byte[] bytes = new byte[16384]; 
                StringBuilder clientData = new StringBuilder();

                // get the stream to talk to the client over
                using (NetworkStream ns = newClient.GetStream())
                {
                    // set initial read timeout to 1 minute to allow for connection
                    ns.ReadTimeout = 60000;
                    // Loop to receive all the data sent by the client.
                    int bytesRead = 0;
                    do
                    {
                        // read the data
                        try
                        {
                            bytesRead = ns.Read(bytes, 0, bytes.Length);
                            if (bytesRead > 0)
                            {
                                // Translate data bytes to an ASCII string and append
                                clientData.Append(
                                    Encoding.ASCII.GetString(bytes, 0, bytesRead));
                                // decrease read timeout to 1 second now that data is coming in
                                ns.ReadTimeout = 1000;
                            }
                        }
                        catch (IOException ioe)
                        {
                            // read timed out, all data has been retrieved
                            if (OnError != null)
                                OnError(ns, "Read timed out: " + ioe.ToString());
                            bytesRead = 0;
                        }
                    } while (bytesRead > 0 && ns.DataAvailable );

                    if (OnDataRecieved != null)
                        OnDataRecieved(ns, clientData.ToString());

                }
            }
            finally
            {
                // stop talking to client
                if (newClient != null)
                    newClient.Close();
            }
        }
        #endregion
    }
}



